/*
 * Test Harness: Demonstrates 2 "features"
 *
 * 1: Pressing the "Rect Sel" button creates a rectangular selection... 
 *    Observe that it is not all that rectangular - it doesn't traverse short or
 *    empty lines neatly.
 *
 * 2: On linux, dragging the window by the title bar causes the selection to grey out because
 *    the text_buffer has lost focus. On OSX and Win32 the text_buffer retains the focus in
 *    this test...
 *
 * Compliation: Just "fltk-config --compile rect_sel_tst.cxx"
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <FL/Fl.H>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Text_Editor.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_Button.H>

/****************************************************************/
Fl_Tabs *tabs;

/****************************************************************/
void new_pane (char * lab)
{
	Fl_Group *grp = new Fl_Group(20,30,640-30, 480-40, lab);
	Fl_Text_Buffer *txt_buf = new Fl_Text_Buffer();
	Fl_Text_Editor *edt = new Fl_Text_Editor(20, 40, 640-40, 480-90);
	grp->end();

	edt->buffer(txt_buf);
	edt->textfont(FL_COURIER);
	txt_buf->text("A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A short line\n"
	              "A short line\n"
				  "\n"
				  "\n"
	              "A short line\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	              "A longish line of text and so forth...\n"
	               );

	tabs->add(grp);
}

/****************************************************************/
void sel_tab(Fl_Widget * w, void *v)
{
    Fl_Group *grp = (Fl_Group *)tabs->value();
	Fl_Text_Editor *edt = (Fl_Text_Editor *)grp->child(0);
    Fl_Text_Buffer *txt_buf = edt->buffer();
	
	int start = txt_buf->line_start(110);
	int end = txt_buf->skip_lines(start, 14);
	
	txt_buf->select_rectangular(start, end, 10, 20);
}

/****************************************************************/
void quit_cb(Fl_Widget * , void *)
{
	exit(0);
}

/****************************************************************/
void Tabs_CB(Fl_Widget *w, void *v)
{
	Fl_Tabs  *tab = (Fl_Tabs *) v;
	Fl_Group *frame = (Fl_Group *)tab->value();

	Fl_Text_Editor *edit = (Fl_Text_Editor*)frame->child(0);
	edit->take_focus();
}

/****************************************************************/
int main(void)
{
	Fl_Window *win = new Fl_Double_Window(640, 530);
	tabs = new Fl_Tabs(10,10, 640-20, 480-20);
	tabs->end();
	tabs->clear_visible_focus();

	Fl_Button * btn = new Fl_Button(10, 480, 60, 30, "Rect Sel");
	Fl_Button * ex  = new Fl_Button(80, 480, 60, 30, "Exit");
	win->end();

	win->resizable(tabs);
	win->show();

	tabs->callback(Tabs_CB, (void*)tabs);
	btn->callback(sel_tab, (void *)win);
	ex->callback(quit_cb, (void *)win);

	new_pane("foo");
	new_pane("bar");
	new_pane("pop");

	return(Fl::run());
}
/****************************************************************/
/* End of File */
